==> racer-0.1.1.tgz: package/lib/adapters/lookup.js <==
var create, createArray, createObject, lookup, lookupAddPath, lookupSetVersion, lookupWithVersion, _ref;
_ref = require('../specHelper'), create = _ref.create, createObject = _ref.createObject, createArray = _ref.createArray;
lookup = function(path, data, getRef) {
  var curr, dereffedPath, i, index, key, keyObj, len, prop, props, refObj;
  curr = data.world;
  props = path.split('.');
  path = '';
  data.$remainder = '';
  i = 0;
  len = props.length;
  while (i < len) {
    prop = props[i++];
    curr = curr[prop];
    path = path ? path + '.' + prop : prop;
    if (curr == null) {
      data.$remainder = props.slice(i).join('.');
      break;
    }
    if (curr.$r) {
      if (getRef && i === len) {
        break;
      }
      refObj = lookup(curr.$r, data);
      dereffedPath = data.$remainder ? "" + data.$path + "." + data.$remainder : data.$path;
      if (key = curr.$k) {
        if (Array.isArray(keyObj = lookup(key, data))) {
          if (i < len) {
            prop = keyObj[props[i++]];
            path = dereffedPath + '.' + prop;
            curr = lookup(path, data);
          } else {
            curr = (function() {
              var _i, _len, _results;
              _results = [];
              for (_i = 0, _len = keyObj.length; _i < _len; _i++) {
                index = keyObj[_i];
                _results.push(lookup(dereffedPath + '.' + index, data));
              }
              return _results;
            })();
          }
        } else {
          dereffedPath += '.' + keyObj;
          curr = lookup(dereffedPath, data);
          if (i !== len) {
            path = dereffedPath;
          }
        }
      } else {
        curr = refObj;
        if (i !== len) {
          path = dereffedPath;
        }
      }
      if (curr == null) {
        data.$remainder = props.slice(i).join('.');
        break;
      }
    }
  }
  data.$path = path;
  return curr;
};
lookupWithVersion = function(path, data, vers) {
  var curr, currVer, dereffedPath, i, index, key, keyObj, len, prop, props, refObj, _ref2, _ref3;
  curr = data.world;
  currVer = vers;
  props = path.split('.');
  path = '';
  data.$remainder = '';
  i = 0;
  len = props.length;
  while (i < len) {
    prop = props[i++];
    curr = curr[prop];
    currVer = currVer[prop] || currVer;
    path = path ? path + '.' + prop : prop;
    if (curr == null) {
      data.$remainder = props.slice(i).join('.');
      break;
    }
    if (curr.$r) {
      _ref2 = lookupWithVersion(curr.$r, data, vers), refObj = _ref2[0], currVer = _ref2[1];
      dereffedPath = data.$remainder ? "" + data.$path + "." + data.$remainder : data.$path;
      if (key = curr.$k) {
        if (Array.isArray(keyObj = lookup(key, data))) {
          if (i < len) {
            prop = keyObj[props[i++]];
            path = dereffedPath + '.' + prop;
            _ref3 = lookupWithVersion(path, data, vers), curr = _ref3[0], currVer = _ref3[1];
          } else {
            curr = (function() {
              var _i, _len, _results;
              _results = [];
              for (_i = 0, _len = keyObj.length; _i < _len; _i++) {
                index = keyObj[_i];
                _results.push(lookup(dereffedPath + '.' + index, data));
              }
              return _results;
            })();
          }
        } else {
          dereffedPath += '.' + keyObj;
          curr = lookup(dereffedPath, data);
          if (i !== len) {
            path = dereffedPath;
          }
        }
      } else {
        curr = refObj;
        if (i !== len) {
          path = dereffedPath;
        }
      }
      if (curr == null) {
        data.$remainder = props.slice(i).join('.');
        break;
      }
    }
  }
  data.$path = path;
  return [curr, currVer];
};
lookupAddPath = function(path, data, setVer, pathType) {
  var curr, dereffedPath, i, index, key, keyObj, len, parent, prop, props, refObj, speculative;
  speculative = setVer == null;
  curr = data.world = speculative ? create(data.world) : data.world;
  props = path.split('.');
  path = '';
  data.$remainder = '';
  i = 0;
  len = props.length;
  while (i < len) {
    prop = props[i++];
    parent = curr;
    curr = curr[prop];
    path = path ? path + '.' + prop : prop;
    if (curr != null) {
      if (speculative && typeof curr === 'object') {
        curr = parent[prop] = create(curr);
      }
    } else {
      if (!pathType) {
        data.$remainder = props.slice(i).join('.');
        break;
      }
      curr = parent[prop] = speculative ? pathType === 'array' && i === len ? createArray() : createObject() : pathType === 'array' && i === len ? [] : {};
    }
    if (curr.$r) {
      refObj = lookupAddPath(curr.$r, data, setVer, pathType);
      dereffedPath = data.$remainder ? "" + data.$path + "." + data.$remainder : data.$path;
      if (key = curr.$k) {
        if (Array.isArray(keyObj = lookup(key, data))) {
          if (i < len) {
            prop = keyObj[props[i++]];
            path = dereffedPath + '.' + prop;
            curr = lookupAddPath(path, data, setVer, pathType);
          } else {
            curr = (function() {
              var _i, _len, _results;
              _results = [];
              for (_i = 0, _len = keyObj.length; _i < _len; _i++) {
                index = keyObj[_i];
                _results.push(lookup(dereffedPath + '.' + index, data));
              }
              return _results;
            })();
          }
        } else {
          dereffedPath += '.' + keyObj;
          curr = lookup(dereffedPath, data);
          if (i !== len) {
            path = dereffedPath;
          }
        }
      } else {
        curr = refObj;
        if (i !== len) {
          path = dereffedPath;
        }
      }
      if (curr == null && !pathType) {
        data.$remainder = props.slice(i).join('.');
        break;
      }
    }
  }
  data.$path = path;
  return curr;
};
lookupSetVersion = function(path, data, vers, setVer, pathType) {
  var curr, currVer, dereffedPath, i, index, key, keyObj, len, parent, prop, props, refObj, speculative, _ref2, _ref3;
  speculative = setVer == null;
  curr = data.world = speculative ? create(data.world) : data.world;
  currVer = vers;
  if (setVer) {
    currVer.ver = setVer = Math.max(setVer, currVer.ver);
  }
  props = path.split('.');
  path = '';
  data.$remainder = '';
  i = 0;
  len = props.length;
  while (i < len) {
    prop = props[i++];
    parent = curr;
    curr = curr[prop];
    currVer = currVer[prop] || (pathType && setVer ? currVer[prop] = {} : currVer);
    path = path ? path + '.' + prop : prop;
    if (curr != null) {
      if (speculative && typeof curr === 'object') {
        curr = parent[prop] = create(curr);
      }
    } else {
      if (!pathType) {
        data.$remainder = props.slice(i).join('.');
        break;
      }
      curr = parent[prop] = speculative ? pathType === 'array' && i === len ? createArray() : createObject() : pathType === 'array' && i === len ? [] : {};
    }
    if (curr.$r) {
      _ref2 = lookupSetVersion(curr.$r, data, vers, setVer, pathType), refObj = _ref2[0], currVer = _ref2[3];
      dereffedPath = data.$remainder ? "" + data.$path + "." + data.$remainder : data.$path;
      if (key = curr.$k) {
        if (Array.isArray(keyObj = lookup(key, data))) {
          if (i < len) {
            prop = keyObj[props[i++]];
            path = dereffedPath + '.' + prop;
            _ref3 = lookupSetVersion(path, data, vers, setVer, pathType), curr = _ref3[0], currVer = _ref3[3];
          } else {
            curr = (function() {
              var _i, _len, _results;
              _results = [];
              for (_i = 0, _len = keyObj.length; _i < _len; _i++) {
                index = keyObj[_i];
                _results.push(lookup(dereffedPath + '.' + index, data));
              }
              return _results;
            })();
          }
        } else {
          dereffedPath += '.' + keyObj;
          curr = lookup(dereffedPath, data);
          if (i !== len) {
            path = dereffedPath;
          }
        }
      } else {
        curr = refObj;
        if (i !== len) {
          path = dereffedPath;
        }
      }
      if (curr == null && !pathType) {
        data.$remainder = props.slice(i).join('.');
        break;
      }
    } else {
      if (setVer) {
        currVer.ver = setVer;
      }
    }
  }
  data.$path = path;
  return [curr, parent, prop, currVer];
};
module.exports = {
  lookup: lookup,
  lookupWithVersion: lookupWithVersion,
  lookupAddPath: lookupAddPath,
  lookupSetVersion: lookupSetVersion
};

==> racer-0.1.3.tgz: package/lib/adapters/lookup.js <==
var create, createArray, createObject, lookup, lookupAddPath, lookupSetVersion, lookupWithVersion, _ref;

_ref = require('../specHelper'), create = _ref.create, createObject = _ref.createObject, createArray = _ref.createArray;

lookup = function(path, data, getRef) {
  var curr, dereffedPath, i, index, key, keyObj, len, prop, props, refObj;
  curr = data.world;
  props = path.split('.');
  path = '';
  data.$remainder = '';
  i = 0;
  len = props.length;
  while (i < len) {
    prop = props[i++];
    curr = curr[prop];
    path = path ? path + '.' + prop : prop;
    if (curr == null) {
      data.$remainder = props.slice(i).join('.');
      break;
    }
    if (curr.$r) {
      if (getRef && i === len) break;
      refObj = lookup(curr.$r, data);
      dereffedPath = data.$remainder ? "" + data.$path + "." + data.$remainder : data.$path;
      if (key = curr.$k) {
        if (Array.isArray(keyObj = lookup(key, data))) {
          if (i < len) {
            prop = keyObj[props[i++]];
            path = dereffedPath + '.' + prop;
            curr = lookup(path, data);
          } else {
            curr = (function() {
              var _i, _len, _results;
              _results = [];
              for (_i = 0, _len = keyObj.length; _i < _len; _i++) {
                index = keyObj[_i];
                _results.push(lookup(dereffedPath + '.' + index, data));
              }
              return _results;
            })();
          }
        } else {
          dereffedPath += '.' + keyObj;
          curr = lookup(dereffedPath, data);
          if (i !== len) path = dereffedPath;
        }
      } else {
        curr = refObj;
        if (i !== len) path = dereffedPath;
      }
      if (curr == null) {
        data.$remainder = props.slice(i).join('.');
        break;
      }
    }
  }
  data.$path = path;
  return curr;
};

lookupWithVersion = function(path, data, vers) {
  var curr, currVer, dereffedPath, i, index, key, keyObj, len, prop, props, refObj, _ref2, _ref3;
  curr = data.world;
  currVer = vers;
  props = path.split('.');
  path = '';
  data.$remainder = '';
  i = 0;
  len = props.length;
  while (i < len) {
    prop = props[i++];
    curr = curr[prop];
    currVer = currVer[prop] || currVer;
    path = path ? path + '.' + prop : prop;
    if (curr == null) {
      data.$remainder = props.slice(i).join('.');
      break;
    }
    if (curr.$r) {
      _ref2 = lookupWithVersion(curr.$r, data, vers), refObj = _ref2[0], currVer = _ref2[1];
      dereffedPath = data.$remainder ? "" + data.$path + "." + data.$remainder : data.$path;
      if (key = curr.$k) {
        if (Array.isArray(keyObj = lookup(key, data))) {
          if (i < len) {
            prop = keyObj[props[i++]];
            path = dereffedPath + '.' + prop;
            _ref3 = lookupWithVersion(path, data, vers), curr = _ref3[0], currVer = _ref3[1];
          } else {
            curr = (function() {
              var _i, _len, _results;
              _results = [];
              for (_i = 0, _len = keyObj.length; _i < _len; _i++) {
                index = keyObj[_i];
                _results.push(lookup(dereffedPath + '.' + index, data));
              }
              return _results;
            })();
          }
        } else {
          dereffedPath += '.' + keyObj;
          curr = lookup(dereffedPath, data);
          if (i !== len) path = dereffedPath;
        }
      } else {
        curr = refObj;
        if (i !== len) path = dereffedPath;
      }
      if (curr == null) {
        data.$remainder = props.slice(i).join('.');
        break;
      }
    }
  }
  data.$path = path;
  return [curr, currVer];
};

lookupAddPath = function(path, data, setVer, pathType) {
  var curr, dereffedPath, i, index, key, keyObj, len, parent, prop, props, refObj, speculative;
  speculative = setVer == null;
  curr = data.world = speculative ? create(data.world) : data.world;
  props = path.split('.');
  path = '';
  data.$remainder = '';
  i = 0;
  len = props.length;
  while (i < len) {
    prop = props[i++];
    parent = curr;
    curr = curr[prop];
    path = path ? path + '.' + prop : prop;
    if (curr != null) {
      if (speculative && typeof curr === 'object') {
        curr = parent[prop] = create(curr);
      }
    } else {
      if (!pathType) {
        data.$remainder = props.slice(i).join('.');
        break;
      }
      curr = parent[prop] = speculative ? pathType === 'array' && i === len ? createArray() : createObject() : pathType === 'array' && i === len ? [] : {};
    }
    if (curr.$r) {
      refObj = lookupAddPath(curr.$r, data, setVer, pathType);
      dereffedPath = data.$remainder ? "" + data.$path + "." + data.$remainder : data.$path;
      if (key = curr.$k) {
        if (Array.isArray(keyObj = lookup(key, data))) {
          if (i < len) {
            prop = keyObj[props[i++]];
            path = dereffedPath + '.' + prop;
            curr = lookupAddPath(path, data, setVer, pathType);
          } else {
            curr = (function() {
              var _i, _len, _results;
              _results = [];
              for (_i = 0, _len = keyObj.length; _i < _len; _i++) {
                index = keyObj[_i];
                _results.push(lookup(dereffedPath + '.' + index, data));
              }
              return _results;
            })();
          }
        } else {
          dereffedPath += '.' + keyObj;
          curr = lookup(dereffedPath, data);
          if (i !== len) path = dereffedPath;
        }
      } else {
        curr = refObj;
        if (i !== len) path = dereffedPath;
      }
      if (curr == null && !pathType) {
        data.$remainder = props.slice(i).join('.');
        break;
      }
    }
  }
  data.$path = path;
  return curr;
};

lookupSetVersion = function(path, data, vers, setVer, pathType) {
  var curr, currVer, dereffedPath, i, index, key, keyObj, len, parent, prop, props, refObj, speculative, _ref2, _ref3;
  speculative = setVer == null;
  curr = data.world = speculative ? create(data.world) : data.world;
  currVer = vers;
  if (setVer) currVer.ver = setVer = Math.max(setVer, currVer.ver);
  props = path.split('.');
  path = '';
  data.$remainder = '';
  i = 0;
  len = props.length;
  while (i < len) {
    prop = props[i++];
    parent = curr;
    curr = curr[prop];
    currVer = currVer[prop] || (pathType && setVer ? currVer[prop] = {} : currVer);
    path = path ? path + '.' + prop : prop;
    if (curr != null) {
      if (speculative && typeof curr === 'object') {
        curr = parent[prop] = create(curr);
      }
    } else {
      if (!pathType) {
        data.$remainder = props.slice(i).join('.');
        break;
      }
      curr = parent[prop] = speculative ? pathType === 'array' && i === len ? createArray() : createObject() : pathType === 'array' && i === len ? [] : {};
    }
    if (curr.$r) {
      _ref2 = lookupSetVersion(curr.$r, data, vers, setVer, pathType), refObj = _ref2[0], currVer = _ref2[3];
      dereffedPath = data.$remainder ? "" + data.$path + "." + data.$remainder : data.$path;
      if (key = curr.$k) {
        if (Array.isArray(keyObj = lookup(key, data))) {
          if (i < len) {
            prop = keyObj[props[i++]];
            path = dereffedPath + '.' + prop;
            _ref3 = lookupSetVersion(path, data, vers, setVer, pathType), curr = _ref3[0], currVer = _ref3[3];
          } else {
            curr = (function() {
              var _i, _len, _results;
              _results = [];
              for (_i = 0, _len = keyObj.length; _i < _len; _i++) {
                index = keyObj[_i];
                _results.push(lookup(dereffedPath + '.' + index, data));
              }
              return _results;
            })();
          }
        } else {
          dereffedPath += '.' + keyObj;
          curr = lookup(dereffedPath, data);
          if (i !== len) path = dereffedPath;
        }
      } else {
        curr = refObj;
        if (i !== len) path = dereffedPath;
      }
      if (curr == null && !pathType) {
        data.$remainder = props.slice(i).join('.');
        break;
      }
    } else {
      if (setVer) currVer.ver = setVer;
    }
  }
  data.$path = path;
  return [curr, parent, prop, currVer];
};

module.exports = {
  lookup: lookup,
  lookupWithVersion: lookupWithVersion,
  lookupAddPath: lookupAddPath,
  lookupSetVersion: lookupSetVersion
};
